/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.tmc.junit.tools;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

public class WarteAufDaten implements ClientReceiverInterface {
	private Collection<SystemObject> _objekte = new LinkedList<SystemObject>();
	private Map<String, ResultData> _daten = new HashMap<String, ResultData>();

	/**
	 * Wartet, bis für die übergebenen Objekte gültige Datensätze der
	 * übergebenen Datenbeschreibung vorliegen
	 * 
	 * @param connection
	 *            die DAV-Verbindung
	 * @param objekte
	 *            die Objekte, für die Datensätze erwartet werden
	 * @param dd
	 *            Datenbeschreibung der erwarteten Datensätze
	 * @param timeout
	 *            Timeout - solange wird maximal auf gültige Daten für alle
	 *            Objekte gewartet - wird hier 0 übergeben, wird unendlich lange
	 *            gewartet (dead lock Gefahr)
	 */
	public WarteAufDaten(ClientDavInterface connection, SystemObject[] objekte, DataDescription dd, long timeout) {
		Collections.addAll(_objekte, objekte);

		connection.subscribeReceiver(this, _objekte, dd, ReceiveOptions.normal(), ReceiverRole.receiver());

		try {
			synchronized (this) {
				if (_daten.size() < _objekte.size()) {
					wait(timeout);
				}
			}
		} catch (InterruptedException e) {
			throw new RuntimeException("Unerwarteter Abbruch", e);
		} finally {
			connection.unsubscribeReceiver(this, _objekte, dd);
		}
	}

	public Map<String, ResultData> getDaten() {
		return Collections.unmodifiableMap(_daten);
	}

	/*
	 * (Kein Javadoc)
	 * 
	 * @see
	 * de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.
	 * main.ResultData[])
	 */
	public void update(ResultData[] resultData) {
		for (ResultData data : resultData) {
			if (data.hasData()) {
				_daten.put(data.getObject().getPid(), data);
			}
		}
		if (_daten.size() >= _objekte.size()) {
			synchronized (this) {
				notify();
			}
		}
	}

}
