/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ste.tmc.situation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.tmc.allgemein.StoerfallZustand;
import de.bsvrz.ste.tmc.parameter.RDSParameterDaten;
import de.bsvrz.ste.tmc.parameter.RDSParameterListener;
import de.bsvrz.ste.tmc.parameter.RDSParameterVerwaltung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Verwaltung von Staus innerhalb eines Objekts vom Typ typ.netz.
 * 
 * 
 * Erstellt auf Basis der SWE RDS/TMC von: Dambach Werke GmbH, Stefan Sans
 * 
 * @since 07/2016: Adaptiert für TMC-Meldungen durch: BitCtrl Systems GmbH
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class VerwaltungStaus extends AbstraktNetzVewDynObj<Situation> implements Runnable, RDSParameterListener {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug DEBUG = Debug.getLogger();

	public static final String MENGE = "Staus";

	public static final String MENGEN_TYP = "typ.verkehrsModellNetz";

	/**
	 * Datenverteiler-Verbindung.
	 */
	private ClientDavInterface _connection;

	/**
	 * @return die Datenverteiler-Verbindung
	 */
	public ClientDavInterface getConnection() {
		return _connection;
	}

	private long _verzoegerung;

	private Collection<SituationEigenschaftenListener> _listener = new ArrayList<SituationEigenschaftenListener>();

	private LinkedHashMap<Stau, Data> _queue = new LinkedHashMap<Stau, Data>();

	private void put(final Stau stau, final Data daten) {
		synchronized (_queue) {
			_queue.remove(stau);
			_queue.put(stau, daten);
			_queue.notify();
		}
	}

	private Map.Entry<Stau, Data> take() throws InterruptedException {
		synchronized (_queue) {
			while (_queue.isEmpty()) {
				_queue.wait();
			}
			final Iterator<Map.Entry<Stau, Data>> it = _queue.entrySet().iterator();
			final Map.Entry<Stau, Data> result = it.next();
			it.remove();
			return result;
		}
	}

	/**
	 * Erzeugt eine neue Stau-Verwaltung.
	 * 
	 * 
	 * @param connection
	 *            {@link ClientDavInterface}
	 * @param netz
	 *            das Konfigurationsobjekt vom Typ typ.netz
	 * @param parameterVerwaltung
	 *            {@link RDSParameterVerwaltung}
	 * @param listener
	 *            Listener
	 * 
	 */
	public VerwaltungStaus(final ClientDavInterface connection, final ConfigurationObject netz,
			final RDSParameterVerwaltung parameterVerwaltung, final SituationEigenschaftenListener[] listener) {
		_connection = connection;

		parameterVerwaltung.addParameterListener(this);
		neueParameter(parameterVerwaltung, parameterVerwaltung.getAktuelleDaten());

		if (listener != null) {
			for (SituationEigenschaftenListener stauEigenschaftenListener : listener) {
				_listener.add(stauEigenschaftenListener);
			}
		}

		final Thread thread = new Thread(this);
		thread.setName("Stau-Glättungs-Thread");
		thread.start();
		DEBUG.config(thread + " gestartet");

		if (!initialisiereNetz(netz, MENGEN_TYP, MENGE)) {
			DEBUG.warning("Das übergebene Netz " + netz.getName() + " enthält KEINE Unternetze vom Typ " + MENGEN_TYP);
		}
	}

	@Override
	protected Situation erzeugeObjekt(final SystemObject obj) {
		final Stau stau = new Stau(getConnection(), obj);

		for (SituationEigenschaftenListener listener : _listener) {
			stau.addSituationsEigenschaftenListener(listener);
		}

		return stau;
	}

	@Override
	protected Situation bereinigeObjekt(final SystemObject obj) {
		new StauBereinigungsThread(obj).start();
		return super.bereinigeObjekt(obj);
	}

	public final int ZYKLUS = 1000;

	/*
	 * (Kein Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		int wartend = 0;

		while (true) {
			try {
				final Map.Entry<Stau, Data> entry = take();
				final Stau stau = entry.getKey();

				synchronized (stau) {
					final Data daten = entry.getValue();

					if (daten == null && stau.getSituationsEigenschaften() != null) {
						// Übergang auf "keine Daten" => glätten
						final long aktuelleZeit = getConnection().getTime();
						if ((aktuelleZeit - stau._zeitstempel) >= (_verzoegerung * 1000)) {
							stau.setSituationEigenschaften(null);
							stau.fireSituationsEigenschaftenListener(null);
						} else { // Daten kommen wieder in die Queue
							put(stau, daten);
							wartend += 1;
						}
					} else if (daten != null) {
						stau.setSituationEigenschaften(daten);
						stau.fireSituationsEigenschaftenListener(new SituationEigenschaften(daten));
					}
				}

				synchronized (_queue) {
					// Falls noch Einträge in der Queue warten, wird die
					// Queue spätestens nach der Zykluszeit wieder geprüft
					if (wartend > 0 && _queue.size() == wartend) {
						_queue.wait(ZYKLUS);
						wartend = 0;
					}
				}
			} catch (final InterruptedException e) {
				DEBUG.warning("Thread wurde unerwartet unterbrochen", e);
			}
		}
	}

	public void neueParameter(final RDSParameterVerwaltung quelle, final RDSParameterDaten daten) {
		_verzoegerung = daten.getRDSVerzoegerungsMatrix(StoerfallZustand.STAU, StoerfallZustand.KEINE_AUSSAGE);
	}

	class Stau extends Situation {
		private long _zeitstempel = 0;

		/**
		 * Konstruiert eine Instanz vom Typ Baustelle und führt die
		 * Empfangsanmeldung durch.
		 * 
		 * @param connection
		 *            Datenverteiler-Verbindung
		 * @param davObjekt
		 *            Systemobjekt vom Typ Situation
		 */
		public Stau(final ClientDavInterface connection, final SystemObject davObjekt) {
			super(connection, davObjekt);
		}

		@Override
		protected void uebernehmeDaten(final ResultData resultData) {
			_zeitstempel = getConnection().getTime();
			put(this, resultData.hasData() ? resultData.getData() : null);
		}
	}

	class StauBereinigungsThread extends Thread {
		StauBereinigungsThread(final SystemObject obj) {
			_obj = obj;
		}

		private SystemObject _obj;

		public void run() {
			final Situation stau = getVerwalteteObjekte().get(_obj.getPid());
			if (stau != null) {
				for (SituationEigenschaftenListener listener : _listener) {
					listener.neueSituationEigenschaften(stau, null);
					stau.removeSituationsEigenschaftenListener(listener);
				}
				stau.dispose();
			}
		}
	}

}
