/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.tmc.test;

import java.util.LinkedHashSet;
import java.util.Set;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Kapselt, initialisiert und bietet Zugriff auf die Daten, die für die RDS/TMC
 * Teststrecke spezifisch sind.
 * 
 * @author Dambach Werke GmbH
 */
public class PrueffallUmgebung {
	/**
	 * Debug-Logger für Logging-Ausgaben.
	 */
	private static final Debug _debug = Debug.getLogger();

	// Konstanten für die PIDs spezifischer Objekte der RDS/TMC-Teststrecke
	// Diese sind bei Bedarf anzupassen
	// public static final String LANDESMELDESTELLE_PID = "bwLandesMeldeStelle";
	public static final String LANDESMELDESTELLE_PID = "lms.test";
	public static final String VERKEHRSMODELLNETZ_PID = "verkehrsModellNetz.testBW";

	public static final long APP_START_VERZOEGERUNG = 30000;

	public static final String SI_VERFAHREN1_PID = "asp.störfallVerfahrenRDS";
	public static final String SI_VERFAHREN2_PID = "asp.störfallVerfahrenStandard";
	public static final String SI_VERFAHREN3_PID = "asp.störfallVerfahrenVKDiffKfz";

	public static final String LOKAL_SI1_PID = "mq.bw.mq_1_hfb1_sw";
	public static final String LOKAL_SI2_PID = "mq.bw.mq_2_hfb1_sw";
	public static final String LOKAL_SI3_PID = "mq.bw.mq_3_hfb1_sw";
	public static final String LOKAL_SI4_PID = "mq.bw.mq_4_hfb1_sw";
	public static final String LOKAL_SI5_PID = "mq.bw.mq_5_hfb1_sw";

	public static final String STRECKEN_SI1_PID = "sts.00003.BY";
	public static final String STRECKEN_SI2_PID = "sts.00004.BY";
	public static final String STRECKEN_SI3_PID = "sts.00005.BY";
	public static final String STRECKEN_SI4_PID = "sts.00006.BY";
	public static final String STRECKEN_SI5_PID = "sts.00007.BY";
	public static final String STRECKEN_SI6_PID = "sts.00008.BY";
	public static final String STRECKEN_SI7_PID = "sts.00009.BY";
	public static final String STRECKEN_SI8_PID = "sts.00010.BY";

	private SystemObject _lms;

	private SystemObject _vmn;

	private SystemObject[] _ass;

	private Aspect[] _verfahren = new Aspect[3];

	private SystemObject[] _mqs = new SystemObject[5];
	private SystemObject[] _abschnitte = new SystemObject[8];

	private StoerfallIndikator[] _indikatorVerfahren1 = new StoerfallIndikator[5];
	private StoerfallIndikator[] _indikatorVerfahren2 = new StoerfallIndikator[5];
	private StoerfallIndikator[] _indikatorVerfahren3 = new StoerfallIndikator[8];

	public PrueffallUmgebung(final ClientDavInterface connection) {
		final DataModel dm = connection.getDataModel();

		// Initialisiere maßgebliche Konfigurationsobjekte
		_lms = dm.getObject(LANDESMELDESTELLE_PID);
		_vmn = dm.getObject(VERKEHRSMODELLNETZ_PID);

		// Initialisiere Verfahren
		_verfahren[0] = dm.getAspect(SI_VERFAHREN1_PID);
		_verfahren[1] = dm.getAspect(SI_VERFAHREN2_PID);
		_verfahren[2] = dm.getAspect(SI_VERFAHREN3_PID);

		// Initialisiere MQs
		_mqs[0] = dm.getObject(LOKAL_SI1_PID);
		_mqs[1] = dm.getObject(LOKAL_SI2_PID);
		_mqs[2] = dm.getObject(LOKAL_SI3_PID);
		_mqs[3] = dm.getObject(LOKAL_SI4_PID);
		_mqs[4] = dm.getObject(LOKAL_SI5_PID);

		final Set<SystemObject> set = new LinkedHashSet<SystemObject>();
		AttributeGroup atg = dm.getAttributeGroup("atg.punktLiegtAufLinienObjekt");
		for (SystemObject mq : _mqs) {
			final Data data = mq.getConfigurationData(atg);
			final SystemObject obj = data.getItem("LinienReferenz").asReferenceValue().getSystemObject();
			set.add(obj);
		}
		_ass = set.toArray(new SystemObject[0]);

		// Initialisiere Abschnitte
		_abschnitte[0] = dm.getObject(STRECKEN_SI1_PID);
		_abschnitte[1] = dm.getObject(STRECKEN_SI2_PID);
		_abschnitte[2] = dm.getObject(STRECKEN_SI3_PID);
		_abschnitte[3] = dm.getObject(STRECKEN_SI4_PID);
		_abschnitte[4] = dm.getObject(STRECKEN_SI5_PID);
		_abschnitte[5] = dm.getObject(STRECKEN_SI6_PID);
		_abschnitte[6] = dm.getObject(STRECKEN_SI7_PID);
		_abschnitte[7] = dm.getObject(STRECKEN_SI8_PID);

		String text = "Benutzte Strassensegmente/TMC-Locationen:";
		atg = dm.getAttributeGroup("atg.äußeresStraßenSegment");
		for (int i = 0; i < _ass.length; i += 1) {
			final Data data = _ass[i].getConfigurationData(atg);
			final String tmcOrt = data.getItem("TmcPrimärOrt").asReferenceValue().getSystemObject().getName();
			final String tmcRichtung = data.getItem("TmcRichtung").asTextValue().getText();
			text += "\n\t" + (i + 1) + ". Straßensegment: " + _ass[i] + " - " + tmcOrt + "/" + tmcRichtung;
		}
		_debug.info(text);

		text = "Benutzte lokale Störfallindikatoren:";
		for (int i = 0; i < _mqs.length; i += 1) {
			text += "\n\t" + (i + 1) + ". Lokaler Störfallindikator: " + _mqs[i];
		}
		_debug.info(text);

		text = "Benutzte streckenbezogene Störfallindikatoren:";
		for (int i = 0; i < _abschnitte.length; i += 1) {
			text += "\n\t" + (i + 1) + ". Streckenbezogener Störfallindikator: " + _abschnitte[i];
		}
		_debug.info(text);

		text = "Benutzte Störfallindikator-Verfahren:";
		text += "\n\tVerfahren 1: " + _verfahren[0].getName();
		text += "\n\tVerfahren 2: " + _verfahren[1].getName();
		text += "\n\tVerfahren 3: " + _verfahren[2].getName();
		_debug.info(text);

		// Melde Indikatoren an
		_indikatorVerfahren1[0] = new StoerfallIndikator(connection, _mqs[0], _verfahren[0]);
		_indikatorVerfahren1[1] = new StoerfallIndikator(connection, _mqs[1], _verfahren[0]);
		_indikatorVerfahren1[2] = new StoerfallIndikator(connection, _mqs[2], _verfahren[0]);
		_indikatorVerfahren1[3] = new StoerfallIndikator(connection, _mqs[3], _verfahren[0]);
		_indikatorVerfahren1[4] = new StoerfallIndikator(connection, _mqs[4], _verfahren[0]);

		_indikatorVerfahren2[0] = new StoerfallIndikator(connection, _mqs[0], _verfahren[1]);
		_indikatorVerfahren2[1] = new StoerfallIndikator(connection, _mqs[1], _verfahren[1]);
		_indikatorVerfahren2[2] = new StoerfallIndikator(connection, _mqs[2], _verfahren[1]);
		_indikatorVerfahren2[3] = new StoerfallIndikator(connection, _mqs[3], _verfahren[1]);
		_indikatorVerfahren2[4] = new StoerfallIndikator(connection, _mqs[4], _verfahren[1]);

		_indikatorVerfahren3[0] = new StoerfallIndikator(connection, _abschnitte[0], _verfahren[2]);
		_indikatorVerfahren3[1] = new StoerfallIndikator(connection, _abschnitte[1], _verfahren[2]);
		_indikatorVerfahren3[2] = new StoerfallIndikator(connection, _abschnitte[2], _verfahren[2]);
		_indikatorVerfahren3[3] = new StoerfallIndikator(connection, _abschnitte[3], _verfahren[2]);
		_indikatorVerfahren3[4] = new StoerfallIndikator(connection, _abschnitte[4], _verfahren[2]);
		_indikatorVerfahren3[5] = new StoerfallIndikator(connection, _abschnitte[5], _verfahren[2]);
		_indikatorVerfahren3[6] = new StoerfallIndikator(connection, _abschnitte[6], _verfahren[2]);
		_indikatorVerfahren3[7] = new StoerfallIndikator(connection, _abschnitte[7], _verfahren[2]);
	}

	public SystemObject getLms() {
		return _lms;
	}

	public SystemObject getVmn() {
		return _vmn;
	}

	public StoerfallIndikator[] getIndikatorVerfahren1() {
		return _indikatorVerfahren1;
	}

	public StoerfallIndikator[] getIndikatorVerfahren2() {
		return _indikatorVerfahren2;
	}

	public StoerfallIndikator[] getIndikatorVerfahren3() {
		return _indikatorVerfahren3;
	}

	public Aspect[] get_verfahren() {
		return _verfahren;
	}

}
